#!/usr/bin/perl -w
#
###########################################################################
#
# fontions.pm
# V 0.1.0 du 03/01/2005
#
###########################################################################
#
#    Copyright (c) Intellique 2004
#    All rights reserved.
#
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.

package fonctions;

use strict;
use base 'Exporter';
our @EXPORT_OK = qw( infosysteme );

my %Infosysteme;			# hash contenant toutes les infos

############################################################################
#                       Fonctions exportes
############################################################################

# rempli le tableau de hash contenant toutes les infos du systeme
sub infosysteme {
    $Infosysteme{'hostname'} = hostname();
    $Infosysteme{'serial_number'} = serial_number();
    $Infosysteme{'cdrom'} = mount_cdrom();
    $Infosysteme{'kernel'} = kernel();
    $Infosysteme{'interfaces'} = interfaces();
    $Infosysteme{'uptime'} = uptime();
    $Infosysteme{'loadavg'} = loadavg();
    $Infosysteme{'cpuinfo'} = cpuinfo();
    $Infosysteme{'PCI'} = pci();
    $Infosysteme{'IDE'} = ide();
    $Infosysteme{'USB'} = usb();
    $Infosysteme{'SCSI'} = scsi();
    $Infosysteme{'meminfo'} = meminfo();

#	partitions();

    $Infosysteme{'filesystems'} = filesystems();

    return %Infosysteme;
}


############################################################################
#                       Sous Fonctions
############################################################################

sub mount_cdrom {
    
    if (-f "/cdrom/STORIQ/storiq.img")
    {
	return "true";
    }
    
    return "false";
}

sub serial_number {
    if (-f "/usr/share/storiq/SerialNumber"){
	my $serial = open_file_line("/usr/share/storiq/SerialNumber");
	
	chomp ($serial);
	
	return $serial;
    }
    return "";
}

sub hostname {
    my $host = qx#/bin/hostname#;
    chomp ($host);
    
    return $host;
}

sub kernel {
    my $file = open_file_line("/proc/version");
    my @champ = split " ", $file;
    my $kernel;
    
    $kernel = $champ[2];
    $kernel .= " smp" if ($file =~ m/smp/i);
  
    $kernel = "N.A." unless ($file);
    
    return $kernel;
} 

sub interfaces () {
    my @file = open_file_tab("/proc/net/dev");
    my %hash;
    
    foreach my $line (@file) {
	if ($line =~ m/(eth.:|lo.*:|sit.:|tr.:|wlan.:)/i) {
	    my $nom = $1;
	    chop $nom;
	    
	    my $addr = qx#/sbin/ifconfig $nom 2> /dev/null#;
	    $hash{$nom}{'Ip'} = "N.A.";
	    $hash{$nom}{'Ip'} = $1 if ($addr =~ /inet adr:(\d+.\d+.\d+.\d+)/);
	    $hash{$nom}{'Ip'} = $1 if ($addr =~ /inet addr:(\d+.\d+.\d+.\d+)/);
	    
	    my ($nul, $ret) = split ":", $line;
	    my @champ = split " ", $ret;
	    
	    $hash{$nom}{'Recoi'} = bytesize($champ[0]);
	    $hash{$nom}{'Envoi'} = bytesize($champ[8]);
	    $hash{$nom}{'Erreur'} = bytesize($champ[2] + $champ[10]);
	    $hash{$nom}{'Drop'} = bytesize($champ[3] + $champ[11]);
	}
    }
    return \%hash;
}

sub uptime {
    my $file = open_file_line("/proc/uptime");
    my @champ = split " ", $file;
    my $uptime;
    
    my $day = $1 if (($champ[0] / 86400) =~ m/(\d*).*/i);
    my $hour = $1 if ((($champ[0] / 3600) - ($day * 24)) =~ m/(\d*).*/i);
    my $min = $1 if ((($champ[0] / 60) - ($day * 24 * 60) - ($hour * 60)) =~ m/(\d*).*/i);
    
    $uptime = "1 jour" if ($day == 1);
    $uptime = $day . " jours" if ($day > 1);
    $uptime .= " " . $hour . " heure" if ($hour <= 1);
    $uptime .= " " . $hour . " heures" if ($hour > 1);
    $uptime .= " " . $min . " minute" if ($min <= 1);
    $uptime .= " " . $min . " minutes" if ($min > 1);
    
    $uptime = "N.A." unless ($file);
    
    return $uptime;
}

sub loadavg {
    my $file = open_file_line("/proc/loadavg");
    my @champ = split " ", $file;
    my $loadavg;
    
    $loadavg = $champ[0] . " " . $champ[1] . " " . $champ[2];
    $loadavg = "N.A. N.A. N.A." unless ($file);
    return $loadavg;
} 

sub cpuinfo {
    my @file = open_file_tab("/proc/cpuinfo");
    
    pop @file;
    my %liste = map {$1 => $2 if ($_ =~ m/(\S+\s?\S+)\s*:\s(.*)/i)} @file;
    
    my %retour;
    $retour{'Processeur'} = grep {$_ if ($_ =~ m/^processor/i)} @file;
    $retour{'Modle'} = $liste{'model name'};
    $retour{'Frquence'} = $liste{'cpu MHz'} . " MHz";
    $retour{'Taille Cache'} = $liste{'cache size'};
    $retour{'Bogomips'} = $liste{'bogomips'};
    
    return "N.A." unless ($file[0]);
    return \%retour;
    
}

sub pci {
    my @lspci = qx#lspci 2> /dev/null | grep -v bridge | grep -v USB#;
    my $i = 0;
    my %PCI;
    
    if ($lspci[0]) {
	foreach my $line (@lspci) {
	    $line =~ s/\S*\s(.*)\n/$1/;
	    $line =~ s/(.*)\s\(rev.*\)/$1/;
	    $PCI{$i++} = $line;
	}
	return \%PCI;
    }
    
    my @file = open_file_tab("/proc/pci");
    shift @file;
    while (@file) {
	next unless ((shift @file) =~ m/^\s*Bus/i);
	
	my $line = shift @file;
	$PCI{$i++} = $1 if (($line !~ m/USB|bridge/) && ($line =~ m/\s*(.*)\s\(rev.*\)\./));
    }
    return \%PCI;
}

sub ide {
    my @liste = qx#ls /proc/ide/ide*/*/model 2> /dev/null#;
    my $i = 0;
    my %IDE;
    
    foreach my $champ (@liste) {
	my ($model, $media, $capacity) = ();
	
	$champ =~ s/model\n//;
	my $nom = $champ;
	$nom =~ s/\/proc\/ide\/ide\w*\///;
	$nom =~ s/\///;
	
	$capacity = "N.A." unless chop ($capacity = open_file_line($champ . "capacity"));
	$media = "N.A." unless chop ($media = open_file_line($champ . "media"));
	$model = "N.A." unless chop ($model = open_file_line($champ . "model"));
	
	$IDE{$i} = $media . ": " . $nom . ": " . $model;
	$IDE{$i} .= " (Capacit: " . (sprintf "%.1f", ($capacity / 2097152)) . " Go)" if ($capacity > 0);
	$i ++;
    }
    return \%IDE;
}

sub usb {
    my @file = open_file_tab("/proc/bus/usb/devices");
    my @temp;
    my $i = 0;
    my %USB;
    
    foreach my $line (@file) {
	push @temp, "" if ($line =~ m/^T:/i);
	$temp[(scalar @temp) - 1] .= "$1 " if ($line =~ m/^S:.*=(.*)\n/);
    }
    
    foreach my $line (@temp) {
	$USB{$i++} = $1 if ($line =~ m/^(.*).{1}$/);
    }
    return \%USB;
}

sub scsi {
    my @file = open_file_tab("/proc/scsi/scsi");
    my @scsi;
    my $i = 0;
    my %SCSI;
    
    return if ($file[0] =~ m/^Attached devices: none/);
    
    while (@file) {
	if ((shift @file) =~ m/Vendor:\s*(.*\S*)\s*Model:\s*(.*\S)\s*Rev/) {
	    my $retour = $1 . " " . $2;
	    $retour .= " (". $1 . ")" if ((shift @file) =~ m/Type:\s*(.*\S)\s*ANSI/);
	    $SCSI{$i++} = $retour;
	}
    }
    return \%SCSI;
} 

sub meminfo {
    my @file = open_file_tab("/proc/meminfo");
    my %meminfo;
    
    if ($file[0]) {
	my %liste = map {$1 => $2 if ($_ =~ m/(\S+\s?\S+)\s*:\s*(.*)/i)} @file;
	
	$meminfo{'MemTotal'} = kbytesize($liste{'MemTotal'});
	$meminfo{'MemFree'} = kbytesize($liste{'MemFree'});
	$liste{'MemTotal'} =~ s/kB//;
	$liste{'MemFree'} =~ s/kB//;
	$liste{'SwapTotal'} =~ s/kB//;
	$liste{'SwapFree'} =~ s/kB//;
	$meminfo{'MemUsed'} = kbytesize($liste{'MemTotal'} - $liste{'MemFree'});
	$meminfo{'MemPurcent'} = (sprintf "%.0f", ( ($liste{'MemTotal'} - $liste{'MemFree'}) * 100/ $liste{'MemTotal'}) ) . "%";
	$meminfo{'SwapTotal'} = kbytesize($liste{'SwapTotal'});
	$meminfo{'SwapFree'} = kbytesize($liste{'SwapFree'});
	$meminfo{'SwapUsed'} = kbytesize($liste{'SwapTotal'} - $liste{'SwapFree'});
	if ($liste{'SwapTotal'} == 0) {
	    $meminfo{'SwapPurcent'} = "0%";
	}
	else {
	    $meminfo{'SwapPurcent'} = (sprintf "%.0f", ( ($liste{'SwapTotal'} - $liste{'SwapFree'}) *100 / $liste{'SwapTotal'}) ) . "%";
	}
    }
    return \%meminfo;
}

sub filesystems {
    my @file = open_file_tab("/proc/mounts");
    return unless ($file[0]);
    
    shift @file;

    my %filesystems;
    
    my @champ;
    my %fstype = map { $champ[1] => $champ[2] if (@champ = split(" ", $_)) } @file;
    
    my @liste = qx#df -kP#;
    shift @liste;
    chop @liste;
    my @fs;
    
    my $total_free = 0;
    my $total_used = 0;
    my $total_size = 0;
    
    foreach my $line (@liste) {
	my ($partition, $size, $used, $free, $purcent, $mount_point) = split(" ", $line);
	
	$filesystems{$mount_point}{'Partition'} = $partition;
	$filesystems{$mount_point}{'Purcent'} = $purcent;
	$filesystems{$mount_point}{'Size'} = kbytesize ($size);
	$filesystems{$mount_point}{'Used'} = kbytesize ($used);
	$filesystems{$mount_point}{'Free'} = kbytesize ($free);
	$filesystems{$mount_point}{'Type'} = $fstype{$mount_point};
		
	$total_free += $free;
	$total_used += $used;
	$total_size += $size;
    }
    
    $filesystems{'total'}{'Purcent'} = (sprintf "%.0f", ($total_used * 100 / $total_size)) . "%";

    $filesystems{'total'}{'Size'} = kbytesize ($total_size);
    $filesystems{'total'}{'Used'} = kbytesize ($total_used);
    $filesystems{'total'}{'Free'} = kbytesize ($total_free);

    return \%filesystems;
}


# sub partitions () {
# 	my %info;
# 	
# 	my %disc = map { m/^D.*\/dev\/([a-z]*):\s*(.*)$/ ? ($1 => $2) : () } `fdisk -l 2> /dev/null`;
# 	return unless ( %disc );
# 
#         foreach my $key (keys %disc) {
#                 $info{$key}{taille} = $2 if ( $disc{$key} =~ m/^([0-9.,]*)\s*\w*,\s*([0-9.,]*)\s*\w*$/ );
#                 $info{$key}{taille} = ($1 * $2 * $3 / 2) if ( $disc{$key} =~ m/^([0-9.,]*)\s*\w*,\s*([0-9.,]*)\s*\w*,\s*([0-9.,]*)\s*\w*$/ );
# 
#                 if ( $key =~ m/hd/ ) {
#                         chomp( $info{$key}{modele} = `cat /proc/ide/$key/model` );
#                 }
#                 else {
#                         chomp( $info{$key}{modele} = `scsi_info /dev/$key | grep MODEL | cut -d '"' -f 2` );
#                 }
# 		
# 		map { m/^(\/dev\/\w*)\s+\S*\s*\d+\s+\d+\s+(\d+)\+?\s+(\w\w?)\s+/i ? ( $self->{structure}{$key}{$3}{$1} = _bytesize($2*1024) ) : () }
# `fdisk -l /dev/$key 2> /dev/null`;
#         }
# 
# 	$Infosysteme{'partitions'} = \%info;
# }
# 


sub bytesize () {
    my $bytes = shift;
    my $result;
    
    if ($bytes > 1099511627776) {
	$result = (sprintf "%.2f", ($bytes / 1073741824)) . " To";
    }
    elsif ($bytes > 1073741824) {
	$result = (sprintf "%.2f", ($bytes / 1073741824)) . " Go";
    }
    elsif ($bytes > 1048576) {
	$result = (sprintf "%.2f", ($bytes / 1048576)) . " Mo";
    }
    elsif ($bytes > 1024) {
	$result = (sprintf "%.2f", ($bytes / 1024)) . " Ko";
    }
    else {
	$result = $bytes;
    }
    
    return $result;
}

sub kbytesize () {
    my $kbytes = shift;
    my $result;

    $kbytes =~ s/kB//;

    if ($kbytes > 1073741824) {
	$result = (sprintf "%.2f", ($kbytes / 1073741824)) . " To";
    }
    elsif ($kbytes > 1048576) {
	$result = (sprintf "%.2f", ($kbytes / 1048576)) . " Go";
    }
    elsif ($kbytes > 1024) {
	$result = (sprintf "%.2f", ($kbytes / 1024)) . " Mo";
    }
    else {
	$result = $kbytes . " Ko";
    }

    return $result;
}

sub open_file_line($) {
    my $path = shift;
    
    if (open (FILES, $path)) {
	my $file = "";
	while (<FILES>) {
	    $file .= $_;
	}
	
	close FILES;
	return $file;
    }
    
#    print ("ERROR $path: $!\n");
    
    return undef;
}

sub open_file_tab($) {
    my $path = shift;
    
    if (open (FILES, $path)) {
	my @file = <FILES>;
	close FILES;
	return @file;
    }
    
#    print ("ERROR $path: $!\n");
    
    return undef;
}

1;
